/**
 * \file peak_detection_granjeaud.cpp
 * \date 5 jun. 2015
 * \author Samuel Granjeaud
 */

#include <cstdio>
#include "peak_detection_zivy.h"
#include "../filters/filter_min_max.h"
#include "../filters/filter_max_min.h"
#include "../filters/filter_smoothing.h"

PeakDetectionZivy::PeakDetectionZivy()
{
	_minmax_half_edge = 3;
	_maxmin_half_edge = 2;
	_detection_threshold_on_minmax = 2000;
	_detection_threshold_on_maxmin = 1000;
	_mean_filter_half_edge = 1;
}

PeakDetectionZivy::~PeakDetectionZivy()
{
}

PeakDetectionBase* PeakDetectionZivy::clone() const {
	qDebug() << "PeakDetectionBase* PeakDetectionZivy::clone() begin";
	PeakDetectionZivy * p_clone = new PeakDetectionZivy();
	p_clone->coeff_detect_range = coeff_detect_range;
	p_clone->_minmax_half_edge = _minmax_half_edge;
	p_clone->_maxmin_half_edge = _maxmin_half_edge;
	p_clone->_detection_threshold_on_minmax = _detection_threshold_on_minmax;
	p_clone->_detection_threshold_on_maxmin = _detection_threshold_on_maxmin;
	p_clone->_mean_filter_half_edge = _mean_filter_half_edge;
	qDebug() << "PeakDetectionBase* PeakDetectionZivy::clone() end";
	
	return p_clone;
}

void
PeakDetectionZivy::set_mean_filter_half_edge(unsigned int mean_filter_half_edge)
{
	_mean_filter_half_edge = mean_filter_half_edge;
}
  
void
PeakDetectionZivy::set_minmax_half_edge(unsigned int minmax)
{
	_minmax_half_edge = minmax;
}
  
void
PeakDetectionZivy::set_maxmin_half_edge(unsigned int maxmin)
{
	_maxmin_half_edge = maxmin;
}
  
void
PeakDetectionZivy::set_detection_threshold_on_max(mcq_double max)
{
	_detection_threshold_on_minmax = max;
}
  
void
PeakDetectionZivy::set_detection_threshold_on_min(mcq_double min)
{
	_detection_threshold_on_maxmin = min;
}


/* NEW
0: median instead average; median is used in all NEW test for smoothing
1: peak intensity must be lower than detection threshold before considering a new peak
2: peak intensity must be lower than half the peak height before considering a new peak
3: smooth again; implemented but buggy
4: short background subtraction; not implemented
Fast implementation of median could be found at
http://ndevilla.free.fr/median/median/
*/
#define NEW 3

void
PeakDetectionZivy::printInfos(QTextStream & out) const
{
	out << "\t detection of type 'zivy' with parameters :\n"
		<< "\t _minmax_half_edge = " << _minmax_half_edge << endl;
	out << "\t _maxmin_half_edge = " << _maxmin_half_edge << endl;
	out << "\t _detection_threshold_on_minmax = "
		<< _detection_threshold_on_minmax << endl;
	out << "\t _detection_threshold_on_maxmin = "
		<< _detection_threshold_on_maxmin << endl;
}

std::vector<xicPeak *> *
PeakDetectionZivy::privNewDetectedPeaks(const QuantiItemBase * quanti_item,
										const Msrun * p_msrun, 
										const std::vector<mcq_double> * p_v_rt,
										const std::vector<mcq_double> * p_v_intensity) const 
{
	
	qDebug() << "PeakDetectionZivy::privNewDetectedPeaks begin";
	std::vector<xicPeak *> * result = new std::vector<xicPeak *>;
	std::vector<mcq_double> * p_v_rtbis = new std::vector<mcq_double>(*p_v_rt);
	std::vector<mcq_double> * p_v_minmax = new std::vector<mcq_double>(*p_v_intensity);
	std::vector<mcq_double> * p_v_maxmin = NULL;
#ifdef NEW_DEBUG
	std::vector<mcq_double> * p_v_smooth = NULL;// only for debugging
#endif
	
	qDebug() << "PeakDetectionZivy::privNewDetectedPeaks 1";
	if (_mean_filter_half_edge != 0)
	{
		// Smooth signal with a smoothing (moving-average) filter before 
		// performing detection. The original signal is not altered.
#ifdef NEW
		FilterSmoothing f_median_smoothing;
		f_median_smoothing.set_smoothing_half_window_length(_mean_filter_half_edge);
		f_median_smoothing.treatSignal(p_v_rtbis, &p_v_minmax);
#else
		FilterSmoothing f_mean_smoothing;
		f_mean_smoothing.set_smoothing_half_window_length(_mean_filter_half_edge);
		f_mean_smoothing.treatSignal(p_v_rtbis, &p_v_minmax);
#endif
		p_v_maxmin = new std::vector<mcq_double>(*p_v_minmax);
	}
	else
	{
		p_v_maxmin = new std::vector<mcq_double>(*p_v_intensity);
	}
#ifdef NEW_DEBUG
	// only for debugging
	p_v_smooth = new std::vector<mcq_double>(*p_v_maxmin);
#endif
	
	qDebug() << "PeakDetectionZivy::privNewDetectedPeaks Close transform ";
	// Close transform 
	FilterMinMax f_minmax;
	f_minmax.set_min_max_half_window_length(_minmax_half_edge);
	f_minmax.treatSignal(p_v_rtbis, & p_v_minmax);
	
	qDebug() << "PeakDetectionZivy::privNewDetectedPeaks Open transform ";
	// Open transform
	FilterMaxMin f_maxmin;
	f_maxmin.set_max_min_half_window_length(_maxmin_half_edge);
	f_maxmin.treatSignal(p_v_rtbis, & p_v_maxmin);
	
	//detect peak positions on close curve : a peak is an intensity value 
	//strictly greater than the two surrounding values. In case of 
	//equality (very rare, can happen with some old old spectrometers) we 
	//take the last equal point to be the peak
	
	std::vector<mcq_double> & v_minmax = *p_v_minmax; //"close" courbe du haut
	std::vector<mcq_double> & v_maxmin = *p_v_maxmin; //"open" courbe du bas
	std::vector<mcq_double> peak_rt_position;

#ifdef NEW_DEBUG
	// DEBUG the peak of interest
	for (unsigned int i = 0; i < v_minmax.size(); i++)
		if (v_minmax[i] != v_maxmin[i] && i > 2515 && i < 2540 && abs((*p_v_intensity)[i]-165000)<10000) {
			fprintf(stderr, "Different %d!!!!\n", i);
			unsigned int a = i-39; unsigned int b = i+39;
			if (a<0) a = 0; if (b>v_minmax.size()) b = v_minmax.size()-9;
			for (unsigned int i = a; i < b; i++)
				fprintf(stderr, "%d: %f, %f, %f: %f > %f\n", i, (*p_v_rt)[i], (*p_v_intensity)[i], (*p_v_smooth)[i], v_minmax[i], v_maxmin[i]);
			break;
		}
#endif

	
#ifdef NEW
	for (unsigned int i = 1, count = 0; i < p_v_minmax->size() - 1; )
#else
	for (unsigned int i = 1, count = 0; i < p_v_minmax->size() - 1; )
#endif
	{
		// conditions to have a peak
		if ( (v_minmax[i - 1 - count] < v_minmax[i]) && 
			 (v_minmax[i] > _detection_threshold_on_minmax) &&
			 (v_maxmin[i] > _detection_threshold_on_maxmin) ) 
		{
			// here we test the last condition to have a peak
			
			// no peak case
			if (v_minmax[i] < v_minmax[i + 1])
			{
				++i;
				count = 0;
			} 
			// there is a peak here ! case
			else if (v_minmax[i] > v_minmax[i + 1])
			{
				xicPeak * tmp_peak = new xicPeak(quanti_item, p_msrun);
				unsigned int begin_peak;
				unsigned int end_peak;

#if NEW == 0
				findPeakRangeWalkingFromMax(&begin_peak, &end_peak, i, v_minmax);

#elif NEW == 1 || NEW == 2
	// HERE WE REIMPLEMENT FINDPEAK
	// Exclusion region during boundaries discovery

	unsigned int begin;
	unsigned int end;
	mcq_double my_threshold;
        unsigned int initial_pos = i; // DO NOT USE i BELOW!!!
	std::vector<mcq_double> curve = v_minmax;
        unsigned int pos;
	mcq_double weight;
	mcq_double weighted_sum;
	mcq_double top;
        unsigned int top_pos;
	mcq_double top_threshold;

#if NEW == 1
	my_threshold = _detection_threshold_on_minmax;
#else
	my_threshold = curve[initial_pos] * 0.25;
	if (my_threshold < _detection_threshold_on_minmax)
		my_threshold = _detection_threshold_on_minmax;
#endif

	//find the begining of the peak (where minmax > 0 and don't go up)
	fprintf(stderr, "initial_pos: %d, threshold: %f\n", initial_pos, my_threshold);
	for (begin = initial_pos; begin > 0 ; begin--)
	{
		if ((curve[begin] < curve[begin-1]) &&
		    (curve[begin] < my_threshold)) break;
#if NEW == 2
		// Update peak height
		if (my_threshold < curve[begin] * 0.25)
			my_threshold = curve[begin] * 0.25;
#endif
	}
	//find the end of the peak 
	for (end = initial_pos; end + 1 < curve.size(); end++)
	{
		if ((curve[end] < curve[end + 1]) &&
		    (curve[end] < my_threshold)) break;
#if NEW == 2
		// Update peak height
		if (my_threshold < curve[end] * 0.25)
			my_threshold = curve[end] * 0.25;
#endif
	}
	fprintf(stderr, "initial_pos: %d, threshold: %f\n", initial_pos, my_threshold);
	fprintf(stderr, "initial_pos: %d, begin_peak: %d, end_peak: %d\n", initial_pos, begin, end);
	//NB: threshold is changed independantly at left and at right
	//thus threshold could be different when computing peak's ends
	//find the top of the peak. This is not corrected yet.
	// find top of the curve
	top_pos = begin; top = curve[top_pos];
	weight = weighted_sum = 0;
	for (pos = begin; pos <= end; pos++)
	{
		weight += curve[pos];
		weighted_sum += curve[pos] * (pos - begin);
		if (top < curve[pos]) {
			top = curve[pos]; top_pos = pos;
		}
	}
	fprintf(stderr, "top_pos: %d, top: %f\n", top_pos, top);
	//pos = floor(weighted_sum/weight) + begin;
	//refine position using intensity greater than X% of the max
	top_threshold = top * 0.70;
	//a minimum number of measures is needed should be used
	//the following is a partial solution
	if (top_threshold < _detection_threshold_on_minmax)
		top_threshold = _detection_threshold_on_minmax;
	weight = weighted_sum = 0;
	for (pos = begin; pos <= end; pos++)
	{
		if (curve[pos] < top_threshold) continue;
		weight += curve[pos];
		weighted_sum += curve[pos] * (pos - begin);
	}
	pos = floor(weighted_sum/weight) + begin;
	fprintf(stderr, "initial_pos: %d, pos: %d\n", initial_pos, pos);
	//HERE WE SIMULATE THE RETURN OF THE FINDPEAK FUNCTION
	i = pos; // SET i!!!
	begin_peak = begin;
	end_peak = end;

#elif NEW == 3
	// Smooth the curve again
	FilterSmoothing f_median_smoothing;
	f_median_smoothing.set_smoothing_half_window_length(5);
	std::vector<mcq_double> * p_v_minmaxbis = new std::vector<mcq_double>(v_minmax);
	f_median_smoothing.treatSignal(p_v_rtbis, &p_v_minmaxbis);
	findPeakRangeWalkingFromMax(&begin_peak, &end_peak, i, *p_v_minmaxbis);

#elif NEW == 4
	// Apply a background on the intensity curve before anything
	// Tests show that it reduces the extension of the peak
	// and therefore make peaks more similar in shape and width
	// This could be useful for top peak detection

#else
				// ORIGINAL
				findPeakRangeWalkingFromMax(&begin_peak, &end_peak, i, v_minmax);
				// ORIGINAL
#endif

#ifdef NEW
				// Quantitation on a fixed number of scans
				unsigned int begin_peak2 = i - 3;
				unsigned int end_peak2   = i + 3;
				mcq_double integr_bounds = 0;
				fprintf(stderr, "%d: %d, %d\n", i, begin_peak, end_peak);
				if (begin_peak2 < begin_peak) begin_peak2 = begin_peak;
				if (begin_peak2 > begin_peak) {
					integr_bounds += (p_v_rt->at(begin_peak2) - p_v_rt->at(begin_peak2-1)) * (p_v_intensity->at(begin_peak2) + 0.001) / 2;
				}
				if (end_peak2 > end_peak)     end_peak2 = end_peak;
				if (end_peak2 < end_peak) {
					integr_bounds += (p_v_rt->at(end_peak2+1) - p_v_rt->at(end_peak2)) * (p_v_intensity->at(end_peak2) + 0.001) / 2;
				}
				mcq_double integr_period = p_v_rt->at(end_peak2)
                                                           - p_v_rt->at(begin_peak2);
				integr_bounds = -integr_bounds / integr_period;
#endif

				// integrate peak surface :
				for (unsigned int j = begin_peak; j <= end_peak; j++)
				{
#ifdef NEW
					// Il demeure les fluctuations comptabilisées ou non de 0.001 à I
					// ne debut de pic et en fin de pic
					// Il faudrait un setter pour begin, end et area
					if (j < begin_peak2 || j > end_peak2)
						tmp_peak->add_measure(p_v_rt->at(j), 0.001);
					else
						tmp_peak->add_measure(p_v_rt->at(j), p_v_intensity->at(j) / integr_period - integr_bounds);
#else
					tmp_peak->add_measure(p_v_rt->at(j), p_v_intensity->at(j));
#endif
				}
				// put peak in the results vector
				if (begin_peak == end_peak)
				{
					delete tmp_peak;
				}
				//else if ((tmp_peak->) && (tmp_peak->get_area() < 1))
				//{
				//	delete tmp_peak;
				//}
				else
				{
					result->push_back(tmp_peak);
				}
				++i;
				count = 0;
#if defined NEW
	i = end_peak;  // directly skip to end of peak
#endif
			}
			// equality case, skipping equal points
			else
			{
				// while (v_minmax[i] == v_minmax[i + 1]) {
				++i, ++count;
			}
		} 
		// no chance to have a peak at all, continue looping
		else
			++i;
	} //end loop for peaks
	qDebug() << "PeakDetectionZivy::privNewDetectedPeaks end loop for peaks";
	
	// clean_up
	delete p_v_minmax;
	delete p_v_maxmin;
	delete p_v_rtbis;
	return (result);
}
